/* arr.c 2001 Erik Greenwald   simple vertex arrays using OpenGL and SDL
   This software is Public Domain. uNF. Completely without warranty or
   constraint. If it helps ya a lot, a shout-out in a readme file and an
   email pointing me to the product would be just dandy, but not required.
*/

#include <GL/gl.h>	/* GL/gl.h, *NOT* gl\gl.h or GL\gl.h or gl/gl.h 
			 * this is the only correct way. Period. */
#include <SDL.h>
#include <SDL_image.h>

#define FLOATLEN (sizeof(GLfloat))


/* this is an 'interlaced' array, it has no inherent benefit over
 * non-interlaced arrays, I just felt like keeping it all togehter. each row 
 * is a vertex, the first two elements are the texcoord, the last 3 are the 
 * vertex location. */

/* I added in the ability to do both interlaced and seperate arrays. The only
 * changes were right here in defining the arrays as well as down in init()
 * where the array states are bound. */
#define INTERLACED_ARRAYS 1

/*  texcoord  vert3d */
#ifdef INTERLACED_ARRAYS
GLfloat vertices[] = {
    0, 0, -1, -1, 0,
    0, 1, -1, 1, 0,
    1, 1, 1, 1, 0,
    1, 0, 1, -1, 0
};

#else

GLfloat vertices[] = {
    -1, -1, 0,
    -1, 1, 0,
    1, 1, 0,
    1, -1, 0
};

GLfloat texcoords[] = {
    0, 0,
    0, 1,
    1, 1,
    1, 0,
};

#endif

/* this is the index array, it references the elements of vertices[] */
GLuint index[] = { 0, 1, 2, 0, 2, 3 };

static void
display (GLuint * tex)
{
    /*
     * normal opengl stuff 
     */
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glBindTexture (GL_TEXTURE_2D, tex[0]);
    glLoadIdentity ();

    /*
     * push out the object so we can see it 
     */
    glTranslatef (0, 0, -5);

    /*
     * this tells it to draw triangles, 6 vertices (2 triangles), using
     * the index[] array and the array that is currently set as the 
     * vertex and texture (and whatever else) arrays for the actual data.
     * Note that index[] must be unsigned ints with at least 6 
     * values. Otherwise this may crash or produce erratic results. 
     */
    glDrawElements (GL_TRIANGLES, 6, GL_UNSIGNED_INT, index);

    /*
     * and put it on the screen 
     */
    SDL_GL_SwapBuffers ();
}

	/*
	 * input function just tests to see if escape was hit, and quits if
	 * it was 
	 */
void
input_do ()
{
    static SDL_Event ev;

    if (SDL_WaitEvent (&ev) != 0 &&
	ev.type == SDL_KEYDOWN && ev.key.keysym.sym == 27)
	SDL_Quit(), exit (0);
    return;
}

	/*
	 * just like a glut reshape... resets the projection matrix. 
	 */
static void
reshape (int w, int h)
{
    glViewport (0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    glFrustum (-1.33, 1.33, -1, 1, 1.5, 20);
    glMatrixMode (GL_MODELVIEW);
    return;
}


static void
init (GLuint * texid)
{
    SDL_Surface *tex;

    glClearColor (0.0, 0.0, 0.0, 0.0);

    /*
     * Enable both texture coordinates and vertex coordinates for the 
     * vertex array. 
     */
    glEnableClientState (GL_TEXTURE_COORD_ARRAY);
    glEnableClientState (GL_VERTEX_ARRAY);
    /*
     * bind the texture and vertex coordinates to the vertices[] array. Note
     * that this sets the state. Any following vertex array commands will
     * 'automatically' know these locations and strides. If you change
     * these, OpenGL will no longer know these and use other data insetad.
     * It's almost like using globals... 
     */
#ifdef INTERLACED_ARRAYS
    glTexCoordPointer (2, GL_FLOAT, 5 * FLOATLEN, vertices);
    glVertexPointer (3, GL_FLOAT, 5 * FLOATLEN, &vertices[2]);
#else
	/* for non-interlaced arrays, notice that both the strides are 0
	 * and two different arrays are addressed. Stride is only needed if
	 * you interlace. */
    glTexCoordPointer (2, GL_FLOAT, 0, vertices);
    glVertexPointer (3, GL_FLOAT, 0, texcoords);
#endif

    /*
     * load the texture and bind it, use SDL_image to make this trivial. 
     */
    tex = IMG_Load ("img.png");
    glEnable (GL_TEXTURE_2D);
    glEnable (GL_BLEND);
    glGenTextures (1, texid);
    glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
    glBindTexture (GL_TEXTURE_2D, texid[0]);
    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, tex->w, tex->h, 0,
	(tex->format->BytesPerPixel == 3) ? GL_RGB : GL_RGBA,
	GL_UNSIGNED_BYTE, tex->pixels);
    SDL_FreeSurface (tex);

    return;
}

int
main (int argc, char **argv)
{
    GLuint texid;
    int flags = SDL_OPENGL | SDL_DOUBLEBUF;
    if(argc==2 && !strncmp(argv[1],"-f", 3))
	flags |= SDL_FULLSCREEN;

    SDL_Init (SDL_INIT_VIDEO);
    SDL_SetVideoMode (640, 480, 32, flags);

    reshape (640, 480);
    init (&texid);

    atexit(SDL_Quit);	/* ew */

    /*
     * the main execution loop. 
     */
    while (1)
    {
	display (&texid);
	input_do ();		/* escape key forces exit(); */
    }

    return 0;

}
